#include <QParallelAnimationGroup>
#include <QPropertyAnimation>
#include <QGraphicsItem>
#include <QDebug>

#include "trafficlog.h"

TrafficLog::TrafficLog() :
    QList<TrafficLogItem>()
{
}

// Constructs a sequential animation. The outer animation is sequential
// because time is sequential, but the inner animations are parallel.
// So for example, the time between seconds 3 and 4 has many parallel
// animations going on. But the animation sequences between 3 and 4, and 4 and 5
// must be sequential.
QSequentialAnimationGroup* TrafficLog::animation(QMap <QString, CarItem*> map) {
    uint64_t time = -1;
    QParallelAnimationGroup *parallel = new QParallelAnimationGroup();

    QSequentialAnimationGroup *sequential = new QSequentialAnimationGroup();
    for (TrafficLogItem item : *this) {
        // This starts a new time step, so the last timestep should be added
        // to the sequential animation group and a new parallel one started.
        if (item.time() != time) {
            time = item.time();
            sequential->addAnimation(parallel);
            parallel = new QParallelAnimationGroup();
        }

        CarItem *graphic = map[item.name()];
        QPropertyAnimation *anim = new QPropertyAnimation(graphic, "pos");
        anim->setDuration(1000);
        anim->setKeyValueAt(1, item.pos());
        parallel->addAnimation(anim);

        if (item.type() == TrafficLogItem::ARRIVED) {
            QPropertyAnimation *fade =
                    new QPropertyAnimation(graphic, "opacity");
            fade->setDuration(1000);
            fade->setStartValue(1.0);
            fade->setEndValue(0.0);
            parallel->addAnimation(fade);
        }
    }

    sequential->addAnimation(parallel);

    return sequential;
}

// Parse a log item from a string. The parsing is not very robust,
// so do not expect graceful failure.
TrafficLogItem readLogItem (QString str) {
    QStringList words = str.split(' ', QString::SkipEmptyParts);

    uint64_t time = words[0].toULong();
    QString name = words[1];
    TrafficLogItem::LogType type =
            words[2] == "position" ?
                TrafficLogItem::POSITION :
                TrafficLogItem::ARRIVED;
    double x = words[3].toDouble();
    double y = words[4].toDouble();

    return TrafficLogItem (time, name, QPointF(x,y), type);
}

TrafficLog readTrafficLogFile (QFile& file) {
    TrafficLog log;
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        qDebug() << "file failed to open";

    while (!file.atEnd()) {
        QString contents = file.readLine();
        log.push_back(readLogItem (contents));
    }
    return log;
}
